@@ -10,10 +10,11 @@ from azure.storage.blob import BlobClient
 from azure.cosmosdb.table import TableService, TableBatch
 from typing import List
 from multiprocessing.pool import ThreadPool
-from azure.core.exceptions import ResourceExistsError
+from azure.core.exceptions import ResourceExistsError, ResourceNotFoundError
 
 enough_workers_message = "There are already enough active workers"
 workers_long_running = "Workers are active for too long. Going to process as well."
+unhandled_exception_message = "Unhandled exception is appeared."
 state_table = "workerState"
 failure_table = "executionFailures"
 segment_failure_table = "segmentExecutionFailures"
@@ -80,27 +81,38 @@ def singleton(class_):
 
 
 class Worker:
-    def __init__(self, worker_name, create_if_not_exist=True, segment_limit=200, connection_string=None, return_only_source=True):
+    def __init__(
+        self, 
+        worker_name, 
+        create_if_not_exist=True, 
+        segment_limit=200, 
+        connection_string=None, 
+        return_only_source=True
+    ):
         if connection_string is None:
             self.connection_string = os.environ['AzureWebJobsStorage']
         else:
             self.connection_string = connection_string
         self.metadata_table_name = worker_name + "Data"
         self.queue_name = worker_name.lower()
+        self.poison_queue_name = self.queue_name + "-poison"
         self.worker_name = worker_name
         self.segment_limit = segment_limit
         self.return_only_source = return_only_source
 
         self.table_client = TableService(connection_string=self.connection_string)
         self.queue = QueueClient.from_connection_string(conn_str=self.connection_string, queue_name=self.queue_name)
+        self.poison_queue = QueueClient.from_connection_string(conn_str=self.connection_string, queue_name=self.poison_queue_name)
 
         if create_if_not_exist:
             try:
                 self.table_client.create_table(self.metadata_table_name)
+            except ResourceExistsError:
+                pass
+            try:
                 self.queue.create_queue()
             except ResourceExistsError:
                 pass
-
         self.pool = ThreadPool(processes=1)
 
     def exists(self) -> bool:
@@ -135,12 +147,14 @@ class Worker:
         return request_id
         
     def get_result(self, request_id):
+        self._process_poison_messages()
         guid, request_hash = self._decode_id(request_id)
         entity = self.table_client.get_entity(self.metadata_table_name, guid, request_hash, select="PartitionKey,RowKey,Result,Error,StackTrace,State,Offset")
         entity = self._expand_result(entity)
         return TaskResult.create_from_request_entity(entity)
 
     def get_results(self, guid):
+        self._process_poison_messages()
         entities = list(
             self.table_client.query_entities(
                 self.metadata_table_name, 
@@ -151,6 +165,30 @@ class Worker:
         entities = [self._expand_result(e) for e in entities]
         return [TaskResult.create_from_request_entity(entity) for entity in entities]
 
+    def _process_poison_messages(self):
+        try:
+            messages = self.poison_queue.receive_messages()
+            for msg in messages:
+                request_id = base64.decodestring(msg.content.encode()).decode()
+                guid, hash = self._decode_id(request_id)
+                request_entity = {
+                    "PartitionKey": guid,
+                    "RowKey": hash,
+                    "Error": unhandled_exception_message,
+                    "StackTrace": "" 
+                }
+                self.table_client.merge_entity(self.metadata_table_name, request_entity)
+                function_failure_entity = {
+                    "PartitionKey": self.worker_name, 
+                    "RowKey": request_id, 
+                    "FailureMessage": unhandled_exception_message, 
+                    "StackTrace": ""
+                }
+                self.table_client.insert_entity(failure_table, function_failure_entity)
+                self.poison_queue.delete_message(msg)
+        except ResourceNotFoundError:
+            pass
+
     def _expand_result(self, entity):
         if "Result" not in entity:
             return entity
